@@ -21,19 +21,23 @@ extern int semanage_bool_key_create(
 
 extern int semanage_bool_key_extract(
 	semanage_handle_t* handle,
-	semanage_bool_t* boolean, 
+	const semanage_bool_t* boolean, 
 	semanage_bool_key_t** key);
 
 extern void semanage_bool_key_free(
 	semanage_bool_key_t* key);
 
 extern int semanage_bool_compare(
-	semanage_bool_t* boolean, 
-	semanage_bool_key_t* key);
+	const semanage_bool_t* boolean, 
+	const semanage_bool_key_t* key);
+
+extern int semanage_bool_compare2(
+	const semanage_bool_t* boolean,
+	const semanage_bool_t* boolean2);
 	
 /* Name */
 extern const char* semanage_bool_get_name(
-	semanage_bool_t* boolean);
+	const semanage_bool_t* boolean);
 
 extern int semanage_bool_set_name(
 	semanage_handle_t* handle,
@@ -42,7 +46,7 @@ extern int semanage_bool_set_name(
 
 /* Value */
 extern int semanage_bool_get_value(
-	semanage_bool_t* boolean);
+	const semanage_bool_t* boolean);
 
 extern void semanage_bool_set_value(
 	semanage_bool_t* boolean, 
@@ -55,7 +59,7 @@ extern int semanage_bool_create(
 
 extern int semanage_bool_clone(
 	semanage_handle_t* handle,
-	semanage_bool_t* boolean, 
+	const semanage_bool_t* boolean, 
 	semanage_bool_t** bool_ptr);
 
 extern void semanage_bool_free(
