@@ -3,37 +3,26 @@
 #ifndef _SEMANAGE_INTERFACES_LOCAL_H_
 #define _SEMANAGE_INTERFACES_LOCAL_H_
 
-#include <stddef.h>
 #include <semanage/iface_record.h>
 #include <semanage/handle.h>
 
-extern int semanage_iface_add_local(
-	semanage_handle_t* handle,
-	semanage_iface_key_t* key,
-	semanage_iface_t* data);
-
 extern int semanage_iface_modify_local(
 	semanage_handle_t* handle,
-	semanage_iface_key_t* key,
-	semanage_iface_t* data);
-
-extern int semanage_iface_set_local(
-	semanage_handle_t* handle,
-	semanage_iface_key_t* key,
-	semanage_iface_t* data);
+	const semanage_iface_key_t* key,
+	const semanage_iface_t* data);
 
 extern int semanage_iface_del_local(
 	semanage_handle_t* handle,
-	semanage_iface_key_t* key);
+	const semanage_iface_key_t* key);
 
 extern int semanage_iface_query_local(
 	semanage_handle_t* handle,
-	semanage_iface_key_t* key,
+	const semanage_iface_key_t* key,
 	semanage_iface_t** response);
 
 extern int semanage_iface_exists_local(
 	semanage_handle_t* handle,
-	semanage_iface_key_t* key,
+	const semanage_iface_key_t* key,
 	int* response);
 
 extern int semanage_iface_count_local(
@@ -42,13 +31,14 @@ extern int semanage_iface_count_local(
 
 extern int semanage_iface_iterate_local(
 	semanage_handle_t* handle,
-	int (*handler) (semanage_iface_t* record,
-	                void* varg),
+	int (*handler) (
+		const semanage_iface_t* record,
+		void* varg),
 	void* handler_arg);
 
 extern int semanage_iface_list_local(
 	semanage_handle_t* handle,
 	semanage_iface_t*** records,
-	size_t* count);
+	unsigned int* count);
 
 #endif 
