@@ -3,18 +3,17 @@
 #ifndef _SEMANAGE_INTERFACES_POLICY_H_
 #define _SEMANAGE_INTERFACES_POLICY_H_
 
-#include <stddef.h>
 #include <semanage/handle.h> 
 #include <semanage/iface_record.h>
 
 extern int semanage_iface_query(
 	semanage_handle_t* handle,
-	semanage_iface_key_t* key,
+	const semanage_iface_key_t* key,
 	semanage_iface_t** response);
 
 extern int semanage_iface_exists(
 	semanage_handle_t* handle,
-	semanage_iface_key_t* key,
+	const semanage_iface_key_t* key,
 	int* response);
 
 extern int semanage_iface_count(
@@ -23,13 +22,14 @@ extern int semanage_iface_count(
 
 extern int semanage_iface_iterate(
 	semanage_handle_t* handle,
-	int (*handler) (semanage_iface_t* record,
-	                void* varg),
+	int (*handler) (
+		const semanage_iface_t* record,
+		void* varg),
 	void* handler_arg);
 
 extern int semanage_iface_list(
 	semanage_handle_t* handle,
 	semanage_iface_t*** records,
-	size_t* count);
+	unsigned int* count);
 
 #endif
