@@ -3,37 +3,26 @@
 #ifndef _SEMANAGE_PORTS_LOCAL_H_
 #define _SEMANAGE_PORTS_LOCAL_H_
 
-#include <stddef.h>
 #include <semanage/port_record.h>
 #include <semanage/handle.h>
 
-extern int semanage_port_add_local(
-	semanage_handle_t* handle,
-	semanage_port_key_t* key,
-	semanage_port_t* data);
-
 extern int semanage_port_modify_local(
 	semanage_handle_t* handle,
-	semanage_port_key_t* key,
-	semanage_port_t* data);
-
-extern int semanage_port_set_local(
-	semanage_handle_t* handle,
-	semanage_port_key_t* key,
-	semanage_port_t* data);
+	const semanage_port_key_t* key,
+	const semanage_port_t* data);
 
 extern int semanage_port_del_local(
 	semanage_handle_t* handle,
-	semanage_port_key_t* key);
+	const semanage_port_key_t* key);
 
 extern int semanage_port_query_local(
 	semanage_handle_t* handle,
-	semanage_port_key_t* key,
+	const semanage_port_key_t* key,
 	semanage_port_t** response);
 
 extern int semanage_port_exists_local(
 	semanage_handle_t* handle,
-	semanage_port_key_t* key,
+	const semanage_port_key_t* key,
 	int* response);
 
 extern int semanage_port_count_local(
@@ -42,13 +31,14 @@ extern int semanage_port_count_local(
 
 extern int semanage_port_iterate_local(
 	semanage_handle_t* handle,
-	int (*handler) (semanage_port_t* record,
-	                void* varg),
+	int (*handler) (
+		const semanage_port_t* record,
+		void* varg),
 	void* handler_arg);
 
 extern int semanage_port_list_local(
 	semanage_handle_t* handle,
 	semanage_port_t*** records,
-	size_t* count);
+	unsigned int* count);
 
 #endif 
