@@ -3,37 +3,26 @@
 #ifndef _SEMANAGE_USERS_LOCAL_H_
 #define _SEMANAGE_USERS_LOCAL_H_
 
-#include <stddef.h>
 #include <semanage/user_record.h>
 #include <semanage/handle.h>
 
-extern int semanage_user_add_local(
-	semanage_handle_t* handle,
-	semanage_user_key_t* key,
-	semanage_user_t* data);
-
 extern int semanage_user_modify_local(
 	semanage_handle_t* handle,
-	semanage_user_key_t* key,
-	semanage_user_t* data);
-
-extern int semanage_user_set_local(
-	semanage_handle_t* handle,
-	semanage_user_key_t* key,
-	semanage_user_t* data);
+	const semanage_user_key_t* key,
+	const semanage_user_t* data);
 
 extern int semanage_user_del_local(
 	semanage_handle_t* handle,
-	semanage_user_key_t* key);
+	const semanage_user_key_t* key);
 
 extern int semanage_user_query_local(
 	semanage_handle_t* handle,
-	semanage_user_key_t* key,
+	const semanage_user_key_t* key,
 	semanage_user_t** response);
 
 extern int semanage_user_exists_local(
 	semanage_handle_t* handle,
-	semanage_user_key_t* key,
+	const semanage_user_key_t* key,
 	int* response);
 
 extern int semanage_user_count_local(
@@ -42,13 +31,14 @@ extern int semanage_user_count_local(
 
 extern int semanage_user_iterate_local(
 	semanage_handle_t* handle,
-	int (*handler) (semanage_user_t* record,
-	                void* varg),
+	int (*handler) (
+		const semanage_user_t* record,
+		void* varg),
 	void* handler_arg);
 
 extern int semanage_user_list_local(
 	semanage_handle_t* handle,
 	semanage_user_t*** records,
-	size_t* count);
+	unsigned int* count);
 
 #endif 
