@@ -1,19 +1,18 @@
 /* Copyright (C) 2005 Red Hat, Inc. */
 
-#include "boolean_internal.h"
-
-typedef semanage_bool_key_t record_key_t;
-typedef semanage_bool_t record_t;
+struct semanage_bool;
+struct semanage_bool_key;
+typedef struct semanage_bool_key record_key_t;
+typedef struct semanage_bool record_t;
 #define DBASE_RECORD_DEFINED
 
-#include <stddef.h>
-#include <semanage/booleans_policy.h>
+#include "boolean_internal.h"
 #include "handle.h"
 #include "database.h"
 
 int semanage_bool_query(
 	semanage_handle_t* handle,
-	semanage_bool_key_t* key,
+	const semanage_bool_key_t* key,
 	semanage_bool_t** response) {
 
 	dbase_config_t* dconfig = semanage_bool_dbase_policy(handle);
@@ -22,7 +21,7 @@ int semanage_bool_query(
 
 int semanage_bool_exists(
 	semanage_handle_t* handle,
-	semanage_bool_key_t* key,
+	const semanage_bool_key_t* key,
 	int* response) {
 
 	dbase_config_t* dconfig = semanage_bool_dbase_policy(handle);
@@ -39,8 +38,9 @@ int semanage_bool_count(
 
 int semanage_bool_iterate(
 	semanage_handle_t* handle,
-	int (*handler) (semanage_bool_t* record,
-	                void* varg),
+	int (*handler) (
+		const semanage_bool_t* record,
+		void* varg),
 	void* handler_arg) {
 
 	dbase_config_t* dconfig = semanage_bool_dbase_policy(handle);
@@ -50,7 +50,7 @@ int semanage_bool_iterate(
 int semanage_bool_list(
 	semanage_handle_t* handle,
 	semanage_bool_t*** records,
-	size_t* count) {
+	unsigned int* count) {
 
 	dbase_config_t* dconfig = semanage_bool_dbase_policy(handle);
 	return dbase_list(handle, dconfig, records, count);
