@@ -1,9 +1,9 @@
 /* Copyright (C) 2005 Red Hat, Inc. */
 
-#include "boolean_internal.h"
-
-typedef semanage_bool_t record_t;
-typedef semanage_bool_key_t record_key_t;
+struct semanage_bool;
+struct semanage_bool_key;
+typedef struct semanage_bool record_t;
+typedef struct semanage_bool_key record_key_t;
 #define DBASE_RECORD_DEFINED
 
 struct dbase_policydb;
@@ -12,13 +12,10 @@ typedef struct dbase_policydb dbase_t;
 
 #include <sepol/booleans.h>
 #include <semanage/handle.h>
-#include "booleans_policydb.h"
+#include "boolean_internal.h"
 #include "debug.h"
 #include "database_policydb.h"
 
-/* BOOL RECORD: metod table (boolean_record.c) */
-extern record_table_t SEMANAGE_BOOL_RTABLE;
-
 /* BOOLEAN RECRORD (SEPOL): POLICYDB extension: method table */
 record_policydb_table_t SEMANAGE_BOOL_POLICYDB_RTABLE = {
 	.add         = NULL, 
