@@ -24,6 +24,7 @@
 #include <assert.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <stdio_ext.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -31,21 +32,20 @@
 #include <sys/types.h>
 #include <limits.h>
 
-#include "users_file.h"
-#include "seusers_file.h"
-#include "ports_file.h"
-#include "interfaces_file.h"
-#include "booleans_file.h"
-#include "users_policydb.h"
-#include "ports_policydb.h"
-#include "interfaces_policydb.h"
-#include "booleans_policydb.h"
+#include "user_internal.h"
+#include "seuser_internal.h"
+#include "port_internal.h"
+#include "iface_internal.h"
+#include "boolean_internal.h"
+#include "fcontext_internal.h"
+#include "node_internal.h"
 
 #include "debug.h"
 #include "handle.h"
 #include "modules.h"
 #include "direct_api.h"
 #include "semanage_store.h"
+#include "database_policydb.h"
 #include "policy.h"
 
 static void semanage_direct_destroy(semanage_handle_t *sh);
@@ -63,6 +63,7 @@ static int semanage_direct_list(semanage_handle_t *sh,
 				semanage_module_info_t **modinfo, int *num_modules);
 
 static struct semanage_policy_table direct_funcs = {
+	.get_serial = semanage_get_commit_number,
 	.destroy = semanage_direct_destroy,
 	.disconnect = semanage_direct_disconnect,
 	.begin_trans = semanage_direct_begintrans,
@@ -82,7 +83,7 @@ int semanage_direct_is_managed(semanage_handle_t *sh) {
 	if (semanage_check_init(polpath))
 		goto err;
 
-	if (semanage_create_store(sh, 0) < 0) 
+	if (semanage_access_check(sh) < 0) 
 		return 0;
 
 	return 1;
@@ -102,7 +103,11 @@ int semanage_direct_connect(semanage_handle_t *sh) {
 	if (semanage_check_init(polpath))
 		goto err;
 
-	if (semanage_create_store(sh, 1) < 0) 
+	if (sh->create_store)
+		if (semanage_create_store(sh, 1))
+			goto err;
+
+	if (semanage_access_check(sh) < SEMANAGE_CAN_READ) 
 		goto err;
 
 	sh->u.direct.translock_file_fd = -1;
@@ -111,23 +116,67 @@ int semanage_direct_connect(semanage_handle_t *sh) {
 	/* set up function pointers */
 	sh->funcs = &direct_funcs;
 
-        /* Configure object databases */
-	if (user_file_dbase_init(sh, semanage_user_dbase_local(sh)) < 0)
+        /* Object databases: local modifications */
+	if (user_base_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_USERS_BASE_LOCAL),
+		semanage_user_base_dbase_local(sh)) < 0)
 		goto err;
 
-	if (port_file_dbase_init(sh, semanage_port_dbase_local(sh)) < 0)
+	if (user_extra_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_USERS_EXTRA_LOCAL),
+		semanage_user_extra_dbase_local(sh)) < 0)
 		goto err;
 
-	if (iface_file_dbase_init(sh, semanage_iface_dbase_local(sh)) < 0)
+	if (user_join_dbase_init(sh,
+		semanage_user_base_dbase_local(sh),
+		semanage_user_extra_dbase_local(sh),
+		semanage_user_dbase_local(sh)) < 0)
 		goto err;
 
-	if (bool_file_dbase_init(sh, semanage_bool_dbase_local(sh)) < 0)
+	if (port_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_PORTS_LOCAL), 
+		semanage_port_dbase_local(sh)) < 0)
 		goto err;
 
-	if (seuser_file_dbase_init(sh, semanage_seuser_dbase(sh)) < 0)
+	if (iface_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_INTERFACES_LOCAL),
+		semanage_iface_dbase_local(sh)) < 0)
 		goto err;
 
-	if (user_policydb_dbase_init(sh, semanage_user_dbase_policy(sh)) < 0)
+	if (bool_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_BOOLEANS_LOCAL),
+		semanage_bool_dbase_local(sh)) < 0)
+		goto err;
+
+	if (fcontext_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_FC_LOCAL), 
+		semanage_fcontext_dbase_local(sh)) < 0)
+		goto err;
+
+	if (seuser_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_SEUSERS_LOCAL),
+		semanage_seuser_dbase_local(sh)) < 0)
+		goto err;
+
+	if (node_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_NODES_LOCAL),
+		semanage_node_dbase_local(sh)) < 0)
+		goto err;
+
+	/* Object databases: local modifications + policy */
+	if (user_base_policydb_dbase_init(sh, 
+		semanage_user_base_dbase_policy(sh)) < 0)
+		goto err;
+
+	if (user_extra_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_USERS_EXTRA), 
+		semanage_user_extra_dbase_policy(sh)) < 0)
+		goto err;
+
+	if (user_join_dbase_init(sh, 
+		semanage_user_base_dbase_policy(sh),
+		semanage_user_extra_dbase_policy(sh),
+		semanage_user_dbase_policy(sh)) < 0)
 		goto err;
 
 	if (port_policydb_dbase_init(sh, semanage_port_dbase_policy(sh)) < 0)
@@ -139,6 +188,23 @@ int semanage_direct_connect(semanage_handle_t *sh) {
 	if (bool_policydb_dbase_init(sh, semanage_bool_dbase_policy(sh)) < 0)
 		goto err;
 
+	if (fcontext_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_FC), 
+		semanage_fcontext_dbase_policy(sh)) < 0)
+		goto err;
+
+	if (seuser_file_dbase_init(sh, 
+		semanage_fname(SEMANAGE_SEUSERS), 
+		semanage_seuser_dbase_policy(sh)) < 0)
+		goto err;
+
+	if (node_policydb_dbase_init(sh, semanage_node_dbase_policy(sh)) < 0)
+		goto err;
+
+	/* Active kernel policy */
+	if (bool_activedb_dbase_init(sh, semanage_bool_dbase_active(sh)) < 0)
+		goto err;
+
 	return STATUS_SUCCESS;
 
 	err:
@@ -163,22 +229,39 @@ static int semanage_direct_disconnect(semanage_handle_t *sh) {
 		semanage_release_trans_lock(sh);
 	}
 
-	/* Remove object databases */
-	user_file_dbase_release(semanage_user_dbase_local(sh));
+	/* Release object databases: local modifications */
+	user_base_file_dbase_release(semanage_user_base_dbase_local(sh));
+	user_extra_file_dbase_release(semanage_user_extra_dbase_local(sh));
+	user_join_dbase_release(semanage_user_dbase_local(sh));
 	port_file_dbase_release(semanage_port_dbase_local(sh));
 	iface_file_dbase_release(semanage_iface_dbase_local(sh));
 	bool_file_dbase_release(semanage_bool_dbase_local(sh));
-	seuser_file_dbase_release(semanage_seuser_dbase(sh));
+	fcontext_file_dbase_release(semanage_fcontext_dbase_local(sh));
+	seuser_file_dbase_release(semanage_seuser_dbase_local(sh));
+	node_file_dbase_release(semanage_node_dbase_local(sh));
 
-	user_policydb_dbase_release(semanage_user_dbase_policy(sh));
+	/* Release object databases: local modifications + policy */
+	user_base_policydb_dbase_release(semanage_user_base_dbase_policy(sh));
+	user_extra_file_dbase_release(semanage_user_extra_dbase_policy(sh));
+	user_join_dbase_release(semanage_user_dbase_policy(sh));
 	port_policydb_dbase_release(semanage_port_dbase_policy(sh));
 	iface_policydb_dbase_release(semanage_iface_dbase_policy(sh));
 	bool_policydb_dbase_release(semanage_bool_dbase_policy(sh));
+	fcontext_file_dbase_release(semanage_fcontext_dbase_policy(sh));
+	seuser_file_dbase_release(semanage_seuser_dbase_policy(sh));
+	node_policydb_dbase_release(semanage_node_dbase_policy(sh));
+
+	/* Release object databases: active kernel policy */
+	bool_activedb_dbase_release(semanage_bool_dbase_active(sh));
 
 	return 0;
 }
 
 static int semanage_direct_begintrans(semanage_handle_t *sh) {
+	
+	if (semanage_access_check(sh) != SEMANAGE_CAN_WRITE) {
+		return -1;
+	}
 	if (semanage_get_trans_lock(sh) < 0) {
 		return -1;
 	}
@@ -222,7 +305,10 @@ static int parse_module_headers(semanage_handle_t *sh, char *module_data,
 	}
 	sepol_policy_file_free(pf);
 	if (file_type != SEPOL_POLICY_MOD) {
-		ERR(sh, "Data did not represent a module.");
+		if (file_type == SEPOL_POLICY_BASE)
+			ERR(sh, "Received a base module, expected a non-base module.");
+		else
+			ERR(sh, "Data did not represent a module.");
 		return -2;
 	}
 	if ((module_path = semanage_path(SEMANAGE_TMP, SEMANAGE_MODULES)) == NULL) {
@@ -263,7 +349,10 @@ static int parse_base_headers(semanage_handle_t *sh,
 	free(module_name);
 	free(version);
 	if (file_type != SEPOL_POLICY_BASE) {
-		ERR(sh, "Data did not represent a module.");
+		if (file_type == SEPOL_POLICY_MOD)
+			ERR(sh, "Received a non-base module, expected a base module.");
+		else
+			ERR(sh, "Data did not represent a module.");
 		return -2;
 	}
 	return 0;
@@ -305,6 +394,7 @@ static int semanage_write_module(semanage_handle_t *sh,
 		ERR(sh, "Could not open %s for writing.", filename);
 		return -1;
 	}
+	__fsetlocking(outfile, FSETLOCKING_BYCALLER);
 	sepol_policy_file_set_fp(pf, outfile);
 	sepol_policy_file_set_handle(pf, sh->sepolh);
 	retval = sepol_module_package_write(package, pf);
@@ -320,45 +410,174 @@ static int semanage_write_module(semanage_handle_t *sh,
 /********************* direct API functions ********************/
 
 /* Commits all changes in sandbox to the actual kernel policy.
- * Returns 0 on success, -1 on error.
+ * Returns commit number on success, -1 on error.
  */
 static int semanage_direct_commit(semanage_handle_t *sh) {
 	char **mod_filenames = NULL;
-	const char *linked_filename = NULL, *fc_filename = NULL;
+	const char *linked_filename = NULL, *ofilename = NULL;
 	sepol_module_package_t *base = NULL;
 	int retval = -1, num_modfiles = 0, i;
+	sepol_policydb_t* out = NULL;
 
-	/* link all modules in the sandbox to the base module */
-	if (semanage_get_modules_names(sh, &mod_filenames, &num_modfiles) != 0 ||
-	    semanage_verify_modules(sh, mod_filenames, num_modfiles) == -1 ||
-	    semanage_link_sandbox(sh, &base) < 0) {
-		goto cleanup;
-	}
+	/* Declare some variables */
+	int modified, fcontexts_modified, ports_modified, 
+		seusers_modified, users_extra_modified;
+	dbase_config_t* users = semanage_user_dbase_local(sh);
+	dbase_config_t* users_base = semanage_user_base_dbase_local(sh);
+	dbase_config_t* pusers_base = semanage_user_base_dbase_policy(sh);
+	dbase_config_t* users_extra = semanage_user_extra_dbase_local(sh);
+	dbase_config_t* pusers_extra = semanage_user_extra_dbase_policy(sh);
+	dbase_config_t* ports = semanage_port_dbase_local(sh);
+	dbase_config_t* pports = semanage_port_dbase_policy(sh);
+	dbase_config_t* bools = semanage_bool_dbase_local(sh);
+	dbase_config_t* pbools = semanage_bool_dbase_policy(sh);
+	dbase_config_t* ifaces = semanage_iface_dbase_local(sh);
+	dbase_config_t* pifaces = semanage_iface_dbase_policy(sh);
+	dbase_config_t* nodes = semanage_node_dbase_local(sh);
+	dbase_config_t* pnodes = semanage_node_dbase_policy(sh);
+	dbase_config_t* fcontexts = semanage_fcontext_dbase_local(sh);
+	dbase_config_t* pfcontexts = semanage_fcontext_dbase_policy(sh);
+	dbase_config_t* seusers = semanage_seuser_dbase_local(sh);
+	dbase_config_t* pseusers = semanage_seuser_dbase_policy(sh);
 
-	/* write the linked base */
-	if ((linked_filename = semanage_path(SEMANAGE_TMP, SEMANAGE_LINKED)) == NULL ||
-	    semanage_write_module(sh, linked_filename, base) == -1 ||
-	    semanage_verify_linked(sh) != 0) {
+	/* Before we do anything else, flush the join to its component parts.
+	 * This *does not* flush to disk automatically */
+	if (users->dtable->is_modified(users->dbase) &&
+	    users->dtable->flush(sh, users->dbase) < 0)
 		goto cleanup;
-	}
 
-	/* write the linked file contexts template */
-	if ((fc_filename = semanage_path(SEMANAGE_TMP, SEMANAGE_FC_TMPL)) == NULL ||
-	    write_file(sh, fc_filename, sepol_module_package_get_file_contexts(base), sepol_module_package_get_file_contexts_len(base)) == -1) {
-		goto cleanup;
-	}
+	/* Decide if anything was modified */
+	fcontexts_modified = fcontexts->dtable->is_modified(fcontexts->dbase);
+	seusers_modified = seusers->dtable->is_modified(seusers->dbase);
+	users_extra_modified = users_extra->dtable->is_modified(users_extra->dbase);
+	ports_modified = ports->dtable->is_modified(ports->dbase);
+
+	modified = sh->modules_modified;
+	modified |= ports_modified;
+	modified |= users->dtable->is_modified(users_base->dbase);
+	modified |= bools->dtable->is_modified(bools->dbase);
+	modified |= ifaces->dtable->is_modified(ifaces->dbase);
+	modified |= nodes->dtable->is_modified(nodes->dbase);
+
+	/* FIXME: get rid of these, once we support loading the existing policy,
+	 * instead of rebuilding it */
+	modified |= seusers_modified;
+	modified |= fcontexts_modified;
+	modified |= users_extra_modified;
+
+	/* If there were policy changes, or explicitly requested, rebuild the policy */
+	if (sh->do_rebuild || modified) {
+
+		/* =================== Module expansion =============== */
+
+		/* link all modules in the sandbox to the base module */
+		if (semanage_get_modules_names(sh, &mod_filenames, &num_modfiles) != 0 ||
+		    semanage_verify_modules(sh, mod_filenames, num_modfiles) == -1 ||
+		    semanage_link_sandbox(sh, &base) < 0) {
+			goto cleanup;
+		}
 	
-	if (semanage_split_fc(sh)) {
-		goto cleanup;
+		/* write the linked base */
+		if ((linked_filename = semanage_path(SEMANAGE_TMP, SEMANAGE_LINKED)) == NULL ||
+		    semanage_write_module(sh, linked_filename, base) == -1 ||
+		    semanage_verify_linked(sh) != 0) {
+			goto cleanup;
+		}
+
+		/* ==================== File-backed ================== */
+
+		/* File Contexts */
+		if ((ofilename = semanage_path(SEMANAGE_TMP, SEMANAGE_FC_TMPL)) == NULL ||
+		    write_file(sh, ofilename, sepol_module_package_get_file_contexts(base), 
+				sepol_module_package_get_file_contexts_len(base)) == -1) {
+			goto cleanup;
+		}
+
+		if (semanage_split_fc(sh)) 
+			goto cleanup;
+
+		pfcontexts->dtable->drop_cache(pfcontexts->dbase);
+
+		/* Seusers */
+		if (sepol_module_package_get_seusers_len(base)) {
+			if ((ofilename = semanage_path(SEMANAGE_TMP, SEMANAGE_SEUSERS)) == NULL ||
+			    write_file(sh, ofilename, sepol_module_package_get_seusers(base), 
+					sepol_module_package_get_seusers_len(base)) == -1) {
+				goto cleanup;
+			}
+			pseusers->dtable->drop_cache(pseusers->dbase);
+
+		} else {
+			if (pseusers->dtable->clear(sh, pseusers->dbase) < 0)
+				goto cleanup;
+		}
+
+		/* Users_extra */
+		if (sepol_module_package_get_user_extra_len(base)) {
+			if ((ofilename = semanage_path(SEMANAGE_TMP, SEMANAGE_USERS_EXTRA)) == NULL ||
+			    write_file(sh, ofilename, sepol_module_package_get_user_extra(base), 
+					sepol_module_package_get_user_extra_len(base)) == -1) {
+				goto cleanup;
+			}
+			pusers_extra->dtable->drop_cache(pusers_extra->dbase);
+
+		} else {
+			if (pusers_extra->dtable->clear(sh, pusers_extra->dbase) < 0)
+				goto cleanup;
+		}
+
+		/* ==================== Policydb-backed ================ */
+
+		/* Create new policy object, then attach to policy databases
+		 * that work with a policydb */
+		if (semanage_expand_sandbox(sh, base, &out) < 0)
+			goto cleanup;
+
+		dbase_policydb_attach((dbase_policydb_t*) pusers_base->dbase, out);
+		dbase_policydb_attach((dbase_policydb_t*) pports->dbase, out);
+		dbase_policydb_attach((dbase_policydb_t*) pifaces->dbase, out);
+		dbase_policydb_attach((dbase_policydb_t*) pbools->dbase, out);
+		dbase_policydb_attach((dbase_policydb_t*) pnodes->dbase, out);
+
+		/* ============= Apply changes, and verify  =============== */
+
+		if (semanage_base_merge_components(sh) < 0)
+			goto cleanup;
+
+		if (semanage_write_policydb(sh, out) < 0)
+			goto cleanup;
+
+		if (semanage_verify_kernel(sh) != 0)
+			goto cleanup;
 	}
 
-	/* Expand the resulting policy */
-	if (semanage_expand_sandbox(sh, base) < 0)
-		goto cleanup;
+	/* FIXME: else if !modified, but seusers_modified, 
+	 * load the existing policy instead of rebuilding */
 
-	/* Verify policy */
-	if (semanage_verify_kernel(sh) != 0)
-		goto cleanup;
+	/* ======= Post-process: Validate non-policydb components ===== */
+
+	/* Validate local modifications to file contexts.
+	 * Note: those are still cached, even though they've been 
+	 * merged into the main file_contexts. We won't check the 
+	 * large file_contexts - checked at compile time */
+	if (sh->do_rebuild || modified || fcontexts_modified) {
+		if (semanage_fcontext_validate_local(sh, out) < 0)
+			goto cleanup;
+	}
+
+	/* Validate local seusers against policy */
+	if (sh->do_rebuild || modified || seusers_modified) {
+		if (semanage_seuser_validate_local(sh, out) < 0) 
+			goto cleanup;
+	}
+
+	/* Validate local ports for overlap */
+	if (sh->do_rebuild || ports_modified) {
+		if (semanage_port_validate_local(sh) < 0)
+			goto cleanup;
+	}
+
+	/* ================== Write non-policydb components ========= */
 
 	/* Commit changes to components */
 	if (semanage_commit_components(sh) < 0)
@@ -370,8 +589,17 @@ static int semanage_direct_commit(semanage_handle_t *sh) {
 	for (i = 0; mod_filenames != NULL && i < num_modfiles; i++) {
 		free(mod_filenames[i]);
 	}
+
+	/* Detach from policydb, so it can be freed */
+	dbase_policydb_detach((dbase_policydb_t*) pusers_base->dbase);
+	dbase_policydb_detach((dbase_policydb_t*) pports->dbase);
+	dbase_policydb_detach((dbase_policydb_t*) pifaces->dbase);
+	dbase_policydb_detach((dbase_policydb_t*) pnodes->dbase);
+	dbase_policydb_detach((dbase_policydb_t*) pbools->dbase);
+
 	free(mod_filenames);
 	sepol_module_package_free(base);
+	sepol_policydb_free(out);
 	semanage_release_trans_lock(sh);
 
 	/* regardless if the commit was successful or not, remove the
@@ -575,6 +803,7 @@ static int semanage_direct_list(semanage_handle_t *sh,
 			 * report it */
 			continue;
 		}
+		__fsetlocking(fp, FSETLOCKING_BYCALLER);
 		sepol_policy_file_set_fp(pf, fp);
 		if (sepol_module_package_info(pf, &type, &name, &version)) {
 			fclose(fp);
@@ -607,3 +836,14 @@ static int semanage_direct_list(semanage_handle_t *sh,
         }
 	return retval;
 }
+
+int semanage_direct_access_check(semanage_handle_t *sh) {
+	char polpath[PATH_MAX];	
+
+	snprintf(polpath, PATH_MAX, "%s%s", selinux_path(), sh->conf->store_path);
+
+	if (semanage_check_init(polpath))
+		return -1;
+
+	return semanage_store_access_check(sh);
+}
