@@ -34,7 +34,6 @@
 
 struct semanage_handle {
 	int con_id;             /* Connection ID */
-	int policy_serial;      /* Policy serial number at connect time */
 
 	/* Error handling */
 	int msg_level;
@@ -62,7 +61,10 @@ struct semanage_handle {
 	int is_connected;
 	int is_in_transaction;
 	int do_reload;		/* whether to reload policy after commit */
+	int do_rebuild;		/* whether to rebuild policy if there were no changes */
 	int modules_modified;
+	int create_store;	/* whether to create the store if it does not exist
+				 * this will only have an effect on direct connections */
 
 	/* This timeout is used for transactions and waiting for lock
 	   -1 means wait indefinetely
@@ -76,22 +78,47 @@ struct semanage_handle {
 	struct semanage_policy_table* funcs;
 
 	/* Object databases */
-#define DBASE_COUNT      9
+#define DBASE_COUNT      19
 
-#define DBASE_LOCAL_USERS       0
-#define DBASE_LOCAL_PORTS       1
-#define DBASE_LOCAL_INTERFACES  2
-#define DBASE_LOCAL_BOOLEANS    3
-#define DBASE_SEUSERS           4
+/* Local modifications */
+#define DBASE_LOCAL_USERS_BASE  0
+#define DBASE_LOCAL_USERS_EXTRA 1
+#define DBASE_LOCAL_USERS       2
+#define DBASE_LOCAL_PORTS       3
+#define DBASE_LOCAL_INTERFACES  4
+#define DBASE_LOCAL_BOOLEANS    5
+#define DBASE_LOCAL_FCONTEXTS	6
+#define DBASE_LOCAL_SEUSERS     7
+#define DBASE_LOCAL_NODES       8
 
-#define DBASE_POLICY_USERS      5
-#define DBASE_POLICY_PORTS      6
-#define DBASE_POLICY_INTERFACES 7
-#define DBASE_POLICY_BOOLEANS   8
+/* Policy + Local modifications */
+#define DBASE_POLICY_USERS_BASE  9
+#define DBASE_POLICY_USERS_EXTRA 10
+#define DBASE_POLICY_USERS       11
+#define DBASE_POLICY_PORTS       12
+#define DBASE_POLICY_INTERFACES  13
+#define DBASE_POLICY_BOOLEANS    14
+#define DBASE_POLICY_FCONTEXTS   15
+#define DBASE_POLICY_SEUSERS     16
+#define DBASE_POLICY_NODES       17
+
+/* Active kernel policy */
+#define DBASE_ACTIVE_BOOLEANS    18
 	dbase_config_t dbase[DBASE_COUNT];
 };
 
+/* === Local modifications === */
 static inline
+dbase_config_t* semanage_user_base_dbase_local(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_LOCAL_USERS_BASE];
+}
+
+static inline
+dbase_config_t* semanage_user_extra_dbase_local(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_LOCAL_USERS_EXTRA];
+}
+
+static inline 
 dbase_config_t* semanage_user_dbase_local(semanage_handle_t* handle) {
 	return &handle->dbase[DBASE_LOCAL_USERS];
 }
@@ -112,8 +139,30 @@ dbase_config_t* semanage_bool_dbase_local(semanage_handle_t* handle) {
 }
 
 static inline
-dbase_config_t* semanage_seuser_dbase(semanage_handle_t* handle) {
-	return &handle->dbase[DBASE_SEUSERS];
+dbase_config_t* semanage_fcontext_dbase_local(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_LOCAL_FCONTEXTS];
+}
+
+static inline
+dbase_config_t* semanage_seuser_dbase_local(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_LOCAL_SEUSERS];
+}
+
+static inline
+dbase_config_t* semanage_node_dbase_local(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_LOCAL_NODES];
+}
+
+
+/* === Policy + Local modifications === */
+static inline
+dbase_config_t* semanage_user_base_dbase_policy(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_POLICY_USERS_BASE];
+}
+
+static inline
+dbase_config_t* semanage_user_extra_dbase_policy(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_POLICY_USERS_EXTRA];
 }
 
 static inline
@@ -136,5 +185,26 @@ dbase_config_t* semanage_bool_dbase_policy(semanage_handle_t* handle) {
 	return &handle->dbase[DBASE_POLICY_BOOLEANS];
 }
 
+static inline
+dbase_config_t* semanage_fcontext_dbase_policy(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_POLICY_FCONTEXTS];
+}
+
+static inline
+dbase_config_t* semanage_seuser_dbase_policy(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_POLICY_SEUSERS];
+}
+
+static inline
+dbase_config_t* semanage_node_dbase_policy(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_POLICY_NODES];
+}
+
+/* === Active kernel policy === */
+static inline
+dbase_config_t* semanage_bool_dbase_active(semanage_handle_t* handle) {
+	return &handle->dbase[DBASE_ACTIVE_BOOLEANS];
+}
+
 #endif
 
