@@ -1,19 +1,18 @@
 /* Copyright (C) 2005 Red Hat, Inc. */
 
-#include "iface_internal.h"
-
-typedef semanage_iface_key_t record_key_t;
-typedef semanage_iface_t record_t;
+struct semanage_iface;
+struct semanage_iface_key;
+typedef struct semanage_iface_key record_key_t;
+typedef struct semanage_iface record_t;
 #define DBASE_RECORD_DEFINED
 
-#include <stddef.h>
-#include <semanage/interfaces_policy.h>
+#include "iface_internal.h"
 #include "handle.h"
 #include "database.h"
 
 int semanage_iface_query(
 	semanage_handle_t* handle,
-	semanage_iface_key_t* key,
+	const semanage_iface_key_t* key,
 	semanage_iface_t** response) {
 
 	dbase_config_t* dconfig = semanage_iface_dbase_policy(handle);
@@ -22,7 +21,7 @@ int semanage_iface_query(
 
 int semanage_iface_exists(
 	semanage_handle_t* handle,
-	semanage_iface_key_t* key,
+	const semanage_iface_key_t* key,
 	int* response) {
 
 	dbase_config_t* dconfig = semanage_iface_dbase_policy(handle);
@@ -39,8 +38,9 @@ int semanage_iface_count(
 
 int semanage_iface_iterate(
 	semanage_handle_t* handle,
-	int (*handler) (semanage_iface_t* record,
-	                void* varg),
+	int (*handler) (
+		const semanage_iface_t* record,
+		void* varg),
 	void* handler_arg) {
 
 	dbase_config_t* dconfig = semanage_iface_dbase_policy(handle);
@@ -50,7 +50,7 @@ int semanage_iface_iterate(
 int semanage_iface_list(
 	semanage_handle_t* handle,
 	semanage_iface_t*** records,
-	size_t* count) {
+	unsigned int* count) {
 
 	dbase_config_t* dconfig = semanage_iface_dbase_policy(handle);
 	return dbase_list(handle, dconfig, records, count);
