@@ -1,9 +1,9 @@
 /* Copyright (C) 2005 Red Hat, Inc. */
 
-#include "iface_internal.h"
-
-typedef semanage_iface_t record_t;
-typedef semanage_iface_key_t record_key_t;
+struct semanage_iface;
+struct semanage_iface_key;
+typedef struct semanage_iface record_t;
+typedef struct semanage_iface_key record_key_t;
 #define DBASE_RECORD_DEFINED
 
 struct dbase_policydb;
@@ -12,13 +12,10 @@ typedef struct dbase_policydb dbase_t;
 
 #include <sepol/interfaces.h>
 #include <semanage/handle.h>
-#include "interfaces_policydb.h"
+#include "iface_internal.h"
 #include "debug.h"
 #include "database_policydb.h"
 
-/* IFACE RECORD: metod table (iface_record.c) */
-extern record_table_t SEMANAGE_IFACE_RTABLE;
-
 /* INTERFACE RECRORD (SEPOL): POLICYDB extension: method table */
 record_policydb_table_t SEMANAGE_IFACE_POLICYDB_RTABLE = {
 	.add         = NULL,
