@@ -1,3 +1,6 @@
+#ifndef _SEMANAGE_MODULE_INTERNAL_H_
+#define _SEMANAGE_MODULE_INTERNAL_H_
+
 #include <semanage/modules.h>
 #include "dso.h"
 
@@ -5,3 +8,5 @@ hidden_proto(semanage_module_get_name)
 hidden_proto(semanage_module_get_version)
 hidden_proto(semanage_module_info_datum_destroy)
 hidden_proto(semanage_module_list_nth)
+
+#endif
