@@ -1,19 +1,18 @@
 /* Copyright (C) 2005 Red Hat, Inc. */
 
-#include <semanage/port_record.h>
-
-typedef semanage_port_key_t record_key_t;
-typedef semanage_port_t record_t;
+struct semanage_port;
+struct semanage_port_key;
+typedef struct semanage_port_key record_key_t;
+typedef struct semanage_port record_t;
 #define DBASE_RECORD_DEFINED
 
-#include <stddef.h>
-#include <semanage/ports_policy.h>
+#include "port_internal.h"
 #include "handle.h" 
 #include "database.h"
 
 int semanage_port_query(
 	semanage_handle_t* handle,
-	semanage_port_key_t* key,
+	const semanage_port_key_t* key,
 	semanage_port_t** response) {
 
 	dbase_config_t* dconfig = semanage_port_dbase_policy(handle);
@@ -22,7 +21,7 @@ int semanage_port_query(
 
 int semanage_port_exists(
 	semanage_handle_t* handle,
-	semanage_port_key_t* key,
+	const semanage_port_key_t* key,
 	int* response) {
 
 	dbase_config_t* dconfig = semanage_port_dbase_policy(handle);
@@ -39,8 +38,9 @@ int semanage_port_count(
 
 int semanage_port_iterate(
 	semanage_handle_t* handle,
-	int (*handler) (semanage_port_t* record,
-	                void* varg),
+	int (*handler) (
+		const semanage_port_t* record,
+		void* varg),
 	void* handler_arg) {
 
 	dbase_config_t* dconfig = semanage_port_dbase_policy(handle);
@@ -50,7 +50,7 @@ int semanage_port_iterate(
 int semanage_port_list(
 	semanage_handle_t* handle,
 	semanage_port_t*** records,
-	size_t* count) {
+	unsigned int* count) {
 
 	dbase_config_t* dconfig = semanage_port_dbase_policy(handle);
 	return dbase_list(handle, dconfig, records, count);
