@@ -1,9 +1,9 @@
 /* Copyright (C) 2005 Red Hat, Inc. */
 
-#include <semanage/port_record.h>
-
-typedef semanage_port_t record_t;
-typedef semanage_port_key_t record_key_t;
+struct semanage_port;
+struct semanage_port_key;
+typedef struct semanage_port record_t;
+typedef struct semanage_port_key record_key_t;
 #define DBASE_RECORD_DEFINED
 
 struct dbase_policydb;
@@ -12,20 +12,10 @@ typedef struct dbase_policydb dbase_t;
 
 #include <sepol/ports.h>
 #include <semanage/handle.h>
-#include "ports_policydb.h"
+#include "port_internal.h"
 #include "debug.h"
 #include "database_policydb.h"
 
-/* FIXME:  Remove when sepol port interface is stabilized and exported. */
-#define sepol_port_modify NULL
-#define sepol_port_query NULL
-#define sepol_port_count NULL
-#define sepol_port_exists NULL
-#define sepol_port_iterate NULL
-
-/* PORT RECORD: method table (port_record.c) */
-extern record_table_t SEMANAGE_PORT_RTABLE;
-
 /* PORT RECORD (SEPOL): POLICYDB extension : method table */
 record_policydb_table_t SEMANAGE_PORT_POLICYDB_RTABLE = {
 	.add         = NULL, 
