@@ -32,7 +32,6 @@ typedef struct dbase_policydb dbase_t;
 #include "semanage_store.h"
 #include "database_policydb.h"
 #include "handle.h"
-#include "policy.h"
 
 #include <selinux/selinux.h>
 #include <sepol/policydb.h>
@@ -44,6 +43,7 @@ typedef struct dbase_policydb dbase_t;
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <stdio_ext.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -96,7 +96,16 @@ static const char *semanage_sandbox_paths[SEMANAGE_STORE_NUM_PATHS] = {
 	"/homedir_template",
 	"/file_contexts.template",
 	"/commit_num",
-	"/seusers"
+	"/ports.local",
+	"/interfaces.local",
+	"/nodes.local",
+	"/booleans.local",
+	"/file_contexts.local",
+	"/seusers",
+	"/users.local",
+	"/users_extra.local",
+	"/seusers.final",
+	"/users_extra",
 };
 
 /* Initialize the paths to config file, lock files and store root.
@@ -113,7 +122,7 @@ static int semanage_init_paths(const char *root) {
 	for (i = 0; i < SEMANAGE_NUM_FILES; i++) {
 		len = (strlen(semanage_relative_files[i]) + prefix_len);
 		semanage_files[i] = calloc(len + 1, sizeof(char));
-		if (!semanage_files[SEMANAGE_ROOT])
+		if (!semanage_files[i])
 			return -1;
 		sprintf(semanage_files[i], "%s%s%s", root, SEMANAGE_MOD_DIR,
 				  semanage_relative_files[i]); 
@@ -187,6 +196,11 @@ int semanage_check_init(const char *root) {
 	return 0;
 }
 
+/* Given a definition number, return a file name from the paths array */
+const char* semanage_fname(enum semanage_sandbox_defs file_enum) {
+	return semanage_sandbox_paths[file_enum];
+}
+
 /* Given a store location (active/previous/tmp) and a definition
  * number, return a fully-qualified path to that file or directory.
  * The caller must not alter the string returned (and hence why this
@@ -221,6 +235,8 @@ int semanage_create_store(semanage_handle_t *sh, int create) {
 	struct stat sb;
 	int mode_mask = R_OK | W_OK | X_OK;
 	const char *path = semanage_files[SEMANAGE_ROOT];
+	int fd;
+
 	if (stat(path, &sb) == -1) {
 		if (errno == ENOENT && create) {
 			if (mkdir(path, S_IRWXU) == -1) {
@@ -278,32 +294,79 @@ int semanage_create_store(semanage_handle_t *sh, int create) {
 			return -1;
 		}
 	}
+	path = semanage_files[SEMANAGE_READ_LOCK];
+	if (stat (path, &sb) == -1) {
+		if (errno == ENOENT && create) {
+			if ((fd = creat(path, S_IRUSR | S_IWUSR)) == -1) {
+				ERR(sh, "Could not create lock file at %s.", path);
+				return -2;
+			}
+		}
+		else {
+			ERR(sh, "Could not read lock file at %s.", path);
+			return -1;
+		}
+	}
+	else {
+		if (!S_ISREG(sb.st_mode) || access(path, R_OK | W_OK) == -1) {
+			ERR(sh, "Could not access lock file at %s.", path);
+			return -1;
+		}
+	}
 	return 0;
 }
 
+/* returns <0 if the active store cannot be read or doesn't exist
+ * 0 if the store exists but the lock file cannot be written to
+ * SEMANAGE_CAN_READ if the store can be read and the lock file written to
+ * SEMANAGE_CAN_WRITE if the modules directory and binary policy dir can be written to
+ */
+int semanage_store_access_check(semanage_handle_t *sh) {
+	const char *path;
+	int rc = -1;
 
+	/* read access on active store */
+	path = semanage_path(SEMANAGE_ACTIVE, SEMANAGE_TOPLEVEL);
+	if (access(path, R_OK | X_OK) != 0)
+		goto out;
+
+	/* we can read the active store meaning it is managed
+	 * so now we return 0 to indicate no error */
+	rc = 0;
+
+	/* read/write access on lock file required for reading */
+	path = semanage_files[SEMANAGE_READ_LOCK];
+	if (access(path, R_OK | W_OK) != 0)
+		goto out;
+
+	/* everything needed for reading has been checked */
+	rc = SEMANAGE_CAN_READ;
+
+	/* check the modules directory */
+	path = semanage_path(SEMANAGE_ACTIVE, SEMANAGE_MODULES);
+	if (access(path, R_OK | W_OK | X_OK) != 0)
+		goto out;
+
+	rc = SEMANAGE_CAN_WRITE;
+
+out:
+	return rc;
+}
 
 /********************* other I/O functions *********************/
 
 /* Callback used by scandir() to select files. */
 static int semanage_filename_select(const struct dirent *d) {
-	int len;
-
-	/* For some reason XFS doesn't set d_type so we actually have
-	 * to use stat() to decide if it's a directory or not :(
-	 */
-	len = strlen(d->d_name);
-	if (len == 1 && d->d_name[0] == '.')
-		return 0;
-	if (len == 2 && d->d_name[0] == '.' &&
-	    d->d_name[1] == '.')
+	if (d->d_name[0] == '.'
+		  && (d->d_name[1] == '\0'
+		   || (d->d_name[1] == '.' && d->d_name[2] == '\0')))
 		return 0;
 	return 1;
 }
 
 /* Copies a file from src to dst.  If dst already exists then
  * overwrite it.  Returns 0 on success, -1 on error. */
-static int semanage_copy_file(const char *src, const char *dst) {
+static int semanage_copy_file(const char *src, const char *dst, mode_t mode) {
 	int in, out, retval = 0, amount_read, n;
 	char tmp[PATH_MAX];
 	char buf[4192];
@@ -315,8 +378,11 @@ static int semanage_copy_file(const char *src, const char *dst) {
 	if ((in = open(src, O_RDONLY)) == -1) {
 		return -1;
 	}
+
+	if (!mode)
+		mode = S_IRUSR | S_IWUSR;
 	
-	if ((out = open(tmp, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR)) == -1) {
+	if ((out = open(tmp, O_WRONLY | O_CREAT | O_TRUNC, mode)) == -1) {
 		close(in);
 		return -1;
 	}
@@ -364,7 +430,7 @@ static int semanage_copy_dir(const char *src, const char *dst) {
 			}
 		}
 		else if (S_ISREG(sb.st_mode)) {
-			if (semanage_copy_file(path, path2) == -1) {
+			if (semanage_copy_file(path, path2, sb.st_mode) == -1) {
 				goto cleanup;
 			}
 		}
@@ -805,52 +871,53 @@ hidden_def(semanage_reload_policy)
 
 /* This expands the file_context.tmpl file to file_context and homedirs.template */
 int semanage_split_fc(semanage_handle_t *sh) {
-	FILE *file_con;
-	int fc, hd, retval = 0;
+	FILE *file_con = NULL;
+	int fc = -1, hd = -1, retval = -1;
 	char buf[PATH_MAX] = {0};
 
 	/* I use fopen here instead of open so that I can use fgets which only reads a single line */
 	file_con = fopen(semanage_path(SEMANAGE_TMP, SEMANAGE_FC_TMPL), "r");
 	if (!file_con) {
 		ERR(sh, "Could not open %s for reading.", semanage_path(SEMANAGE_TMP, SEMANAGE_FC_TMPL));
-		return -1;
+		goto cleanup;
 	}
 
 	fc = open(semanage_path(SEMANAGE_TMP, SEMANAGE_FC), O_WRONLY | O_CREAT | O_TRUNC,  S_IRUSR | S_IWUSR);
 	if (!fc) {
 		ERR(sh, "Could not open %s for writing.", semanage_path(SEMANAGE_TMP, SEMANAGE_FC));
-		return -1;
+		goto cleanup;
 	}
 	hd = open(semanage_path(SEMANAGE_TMP, SEMANAGE_HOMEDIR_TMPL), O_WRONLY | O_CREAT | O_TRUNC,  S_IRUSR | S_IWUSR);
 	if (!hd) {
 		ERR(sh, "Could not open %s for writing.", semanage_path(SEMANAGE_TMP, SEMANAGE_HOMEDIR_TMPL));
-		close(fc);
-		return -1;
+		goto cleanup;
 	}
 
-	while (fgets(buf, PATH_MAX, file_con)) {
-		if (strstr(buf, "HOME_DIR") ||
-		    strstr(buf, "HOME_ROOT") ||
+	while (fgets_unlocked(buf, PATH_MAX, file_con)) {
+		if (!strncmp(buf, "HOME_DIR", 8) ||
+		    !strncmp(buf, "HOME_ROOT", 9) ||
 		    strstr(buf, "ROLE")) {
 			/* This contains one of the template variables, write it to homedir.template */
 			if (write(hd, buf, strnlen(buf, PATH_MAX)) == 0) {
 				ERR(sh, "Write to %s failed.", semanage_path(SEMANAGE_TMP, SEMANAGE_HOMEDIR_TMPL));
-				retval = -1;
 				goto cleanup;
 			}
 		} else {
 			if (write(fc, buf, strnlen(buf, PATH_MAX)) == 0) {
 				ERR(sh, "Write to %s failed.", semanage_path(SEMANAGE_TMP, SEMANAGE_FC));
-				retval = -1;	
 				goto cleanup;
 			}
 		}
 	}
-	
+
+	retval = 0;
 cleanup:
-	fclose(file_con);
-	close(fc);
-	close(hd);	
+	if (file_con)
+		fclose(file_con);
+	if (fc >= 0)
+		close(fc);
+	if (hd >= 0)
+		close(hd);
 
 	return retval;
 		
@@ -895,25 +962,25 @@ static int semanage_install_active(semanage_handle_t *sh) {
 
 	snprintf(store_pol, PATH_MAX, "%s%s.%d", storepath,
 		 running_policy, sh->conf->policyvers);
-	if (semanage_copy_file(active_kernel, store_pol) == -1) {
+	if (semanage_copy_file(active_kernel, store_pol, sh->conf->file_mode) == -1) {
 		ERR(sh, "Could not copy %s to %s.", active_kernel, store_pol);
 		goto cleanup;
 	}
 
 	snprintf(store_hd, PATH_MAX, "%s%s", storepath, running_hd);	
-	if (semanage_copy_file(active_hd, store_hd) == -1) {
+	if (semanage_copy_file(active_hd, store_hd, sh->conf->file_mode) == -1) {
 		ERR(sh, "Could not copy %s to %s.", active_hd, store_hd);
 		goto cleanup;
 	}
 
 	snprintf(store_fc, PATH_MAX, "%s%s", storepath, running_fc);
-	if (semanage_copy_file(active_fc, store_fc) == -1) {
+	if (semanage_copy_file(active_fc, store_fc, sh->conf->file_mode) == -1) {
 		ERR(sh, "Could not copy %s to %s.", active_fc, store_fc);
 		goto cleanup;
 	}
 
 	snprintf(store_seusers, PATH_MAX, "%s%s", storepath, running_seusers);
-	if (semanage_copy_file(active_seusers, store_seusers) == -1) {
+	if (semanage_copy_file(active_seusers, store_seusers, sh->conf->file_mode) == -1) {
 		INFO(sh, "Non-fatal error:  Could not copy %s to %s.", active_seusers, store_seusers);
 		/* Non-fatal; fall through */
 	}
@@ -1047,10 +1114,10 @@ static int semanage_commit_sandbox(semanage_handle_t *sh) {
  * directory, copy it to the binary policy path, then load it.	Upon
  * error move the active directory back to the sandbox.	 This function
  * should be placed within a mutex lock to ensure that it runs
- * atomically.	Returns 0 on success, -1 on error.
+ * atomically.	Returns commit number on success, -1 on error.
  */
 int semanage_install_sandbox(semanage_handle_t *sh) {
-	int retval = -1;
+        int retval = -1, commit_num = -1;
 
 	if (sh->conf->load_policy == NULL) {
 		ERR(sh, "No load_policy program specified in configuration file.");
@@ -1061,7 +1128,8 @@ int semanage_install_sandbox(semanage_handle_t *sh) {
 		goto cleanup;
 	}
 
-	if ((retval = semanage_commit_sandbox(sh)) < 0) {
+	if ((commit_num = semanage_commit_sandbox(sh)) < 0) {
+		retval = commit_num;
 		goto cleanup;
 	}
 
@@ -1070,6 +1138,7 @@ int semanage_install_sandbox(semanage_handle_t *sh) {
 		goto cleanup;
 	}
 
+	retval = commit_num;
 
 cleanup:
 	return retval;
@@ -1264,6 +1333,7 @@ static int semanage_load_module(semanage_handle_t *sh, const char *filename, sep
 		ERR(sh, "Could not open module file %s for reading.", filename);
 		goto cleanup;
 	}
+	__fsetlocking(fp, FSETLOCKING_BYCALLER);
 	sepol_policy_file_set_fp(pf, fp);
 	sepol_policy_file_set_handle(pf, sh->sepolh);
 	if (sepol_module_package_read(*package, pf, 0) == -1) {
@@ -1315,11 +1385,6 @@ int semanage_link_sandbox(semanage_handle_t *sh,
 	    semanage_load_module(sh, base_filename, base) == -1) {
 		goto cleanup;
 	}
-	if (num_modules == 0) {
-		/* no modules, so skip over the rest of this function */
-		retval = 0;
-		goto cleanup;
-	}
 	if ((mods = calloc(num_modules, sizeof(*mods))) == NULL) {
 		ERR(sh, "Out of memory!");
 		num_modules  = 0;
@@ -1350,47 +1415,51 @@ int semanage_link_sandbox(semanage_handle_t *sh,
 	return retval;
 }
 
-/* Expands the policy contained within *base and write the final
- * policy to the sandbox's kernel policy.  Returns 0 on success, -1 on
- * error.
+/* 
+ * Expands the policy contained within *base 
  */
-int semanage_expand_sandbox(semanage_handle_t *sh, sepol_module_package_t *base) {
-	struct sepol_policydb *out;
-	int retval = -1;
-	const char *kernel_filename = NULL;
-	struct sepol_policy_file *pf = NULL;
+int semanage_expand_sandbox(
+	semanage_handle_t *sh, 
+	sepol_module_package_t *base,
+	sepol_policydb_t** policydb) {
+
+	struct sepol_policydb *out = NULL;
 	int policyvers = sh->conf->policyvers;
 	int expand_check = sh->conf->expand_check ? sh->modules_modified : 0;
-	FILE *outfile = NULL;
 
-	if (sepol_policydb_create(&out)) {
-		return -1;
-	}
+	if (sepol_policydb_create(&out))
+		goto err;
+
 	if (sepol_expand_module(sh->sepolh, 
-				sepol_module_package_get_policy(base), out, 0, expand_check)
-				== -1) {
+			sepol_module_package_get_policy(base), out, 0, expand_check)
+			== -1) {
 		ERR(sh, "Expand module failed");
-		goto cleanup;
+		goto err;
 	}
 	if (sepol_policydb_set_vers(out, policyvers)) {
  		ERR(sh, "Unknown/Invalid policy version %d.", policyvers);
-		goto cleanup;
+		goto err;
 	}
 
-	dbase_policydb_attach(semanage_user_dbase_policy(sh)->dbase, out);
-	dbase_policydb_attach(semanage_port_dbase_policy(sh)->dbase, out);
-	dbase_policydb_attach(semanage_iface_dbase_policy(sh)->dbase, out);
-	dbase_policydb_attach(semanage_bool_dbase_policy(sh)->dbase, out);
+	*policydb = out;
+	return STATUS_SUCCESS;
 
-	retval = semanage_base_merge_components(sh);
+	err:
+	sepol_policydb_free(out);
+	return STATUS_ERR;
+}
 
-	dbase_policydb_detach(semanage_user_dbase_policy(sh)->dbase);
-        dbase_policydb_detach(semanage_port_dbase_policy(sh)->dbase);
-        dbase_policydb_detach(semanage_iface_dbase_policy(sh)->dbase);
-        dbase_policydb_detach(semanage_bool_dbase_policy(sh)->dbase);
+/**
+ * Writes the final policy to the sandbox (kernel)
+ */
+int semanage_write_policydb(
+	semanage_handle_t *sh,
+	sepol_policydb_t* out) {
 
-	if (retval < 0) 
-		goto cleanup;
+	int retval = STATUS_ERR;
+	const char *kernel_filename = NULL;
+	struct sepol_policy_file *pf = NULL;
+	FILE *outfile = NULL;
 
 	if ((kernel_filename = semanage_path(SEMANAGE_TMP, SEMANAGE_KERNEL)) == NULL) {
 		goto cleanup;
@@ -1399,6 +1468,7 @@ int semanage_expand_sandbox(semanage_handle_t *sh, sepol_module_package_t *base)
  		ERR(sh, "Could not open kernel policy %s for writing.", kernel_filename);
 		goto cleanup;
 	}
+	__fsetlocking(outfile, FSETLOCKING_BYCALLER);
 	if (sepol_policy_file_create(&pf)) {
  		ERR(sh, "Out of memory!");
 		goto cleanup;
@@ -1409,13 +1479,12 @@ int semanage_expand_sandbox(semanage_handle_t *sh, sepol_module_package_t *base)
  		ERR(sh, "Error while writing kernel policy to %s.", kernel_filename);
 		goto cleanup;
 	}
-	retval = 0;
+	retval = STATUS_SUCCESS;
 
  cleanup:
 	if (outfile != NULL) {
 		fclose(outfile);
 	}
-	sepol_policydb_free(out);
 	sepol_policy_file_free(pf);
 	return retval;
 }
