@@ -45,7 +45,16 @@ enum semanage_sandbox_defs {
 	SEMANAGE_HOMEDIR_TMPL,
 	SEMANAGE_FC_TMPL,
 	SEMANAGE_COMMIT_NUM_FILE,
+	SEMANAGE_PORTS_LOCAL,
+	SEMANAGE_INTERFACES_LOCAL,
+	SEMANAGE_NODES_LOCAL,
+	SEMANAGE_BOOLEANS_LOCAL,
+	SEMANAGE_FC_LOCAL,
+	SEMANAGE_SEUSERS_LOCAL,
+	SEMANAGE_USERS_BASE_LOCAL,
+	SEMANAGE_USERS_EXTRA_LOCAL,
 	SEMANAGE_SEUSERS,
+	SEMANAGE_USERS_EXTRA,
 	SEMANAGE_STORE_NUM_PATHS
 };
 
@@ -54,15 +63,27 @@ enum semanage_sandbox_defs {
  */
 const char *semanage_conf_path(void);
 int semanage_check_init(const char *root);
-const char *semanage_path(enum semanage_store_defs store, enum semanage_sandbox_defs file);
+
+extern const char* semanage_fname(
+	enum semanage_sandbox_defs file_enum);
+
+extern const char *semanage_path(
+	enum semanage_store_defs store, 
+	enum semanage_sandbox_defs file);
 
 int semanage_create_store(semanage_handle_t *sh, int create);
 
+int semanage_store_access_check(semanage_handle_t *sh);
+
 int semanage_remove_directory(const char *path);
 
 int semanage_make_sandbox(semanage_handle_t *sh);
-int semanage_get_modules_names(semanage_handle_t *sh,
-			       char ***filenames, int *len);
+
+int semanage_get_modules_names(
+	semanage_handle_t *sh,
+	char ***filenames, 
+	int *len);
+
 int semanage_install_sandbox(semanage_handle_t *sh);
 
 /* lock file routines */
@@ -72,13 +93,26 @@ void semanage_release_trans_lock(semanage_handle_t *sh);
 void semanage_release_active_lock(semanage_handle_t *sh);
 int semanage_get_commit_number(semanage_handle_t *sh);
 
+int semanage_link_sandbox(
+	semanage_handle_t *sh, 
+	sepol_module_package_t **base);
+
+int semanage_expand_sandbox(
+	semanage_handle_t *sh, 
+	sepol_module_package_t *base,
+	sepol_policydb_t** policydb);
+
+int semanage_write_policydb(
+	semanage_handle_t *sh,
+	sepol_policydb_t* policydb);
 
-int semanage_link_sandbox(semanage_handle_t *sh, sepol_module_package_t **base);
-int semanage_expand_sandbox(semanage_handle_t *sh, sepol_module_package_t *base);
 int semanage_install_sandbox(semanage_handle_t *sh);
 
-int semanage_verify_modules(semanage_handle_t *sh,
-			    char **module_filenames, int num_modules);
+int semanage_verify_modules(
+	semanage_handle_t *sh,
+	char **module_filenames, 
+	int num_modules);
+
 int semanage_verify_linked(semanage_handle_t *sh);
 int semanage_verify_kernel(semanage_handle_t *sh);
 int semanage_split_fc(semanage_handle_t *sh);
